@@ -1,4 +1,4 @@
-/* $Id: gweather-about.c,v 1.4 2000/02/23 08:14:26 peterh Exp $ */
+/* $Id: gweather-about.c,v 1.5 2002/01/22 23:15:28 kevinv Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -20,6 +20,7 @@
 
 #include <gnome.h>
 
+#include "gweather.h"
 #include "gweather-about.h"
 
 
@@ -27,6 +28,7 @@ void gweather_about_run (void)
 {
     const gchar *authors[] = {
         "Spiros Papadimitriou <spapadim+@cs.cmu.edu>",
+        "Todd Kulesza <fflewddur@dropline.net>",
         NULL
     };
     static GtkWidget *about_dialog = NULL;
@@ -37,15 +39,16 @@ void gweather_about_run (void)
     	gdk_window_raise(about_dialog->window);
 	return;
     }
-    about_dialog = gnome_about_new ("GNOME Weather", "0.05",
+    about_dialog = gnome_about_new (_("GNOME Weather"), VERSION,
                                     _("Copyright (c)1999 by S. Papadimitriou"),
-                                    authors,
                                     _("GNOME weather monitor applet.\nWeb: http://gweather.dhs.org/"),
+                                    authors,
+                                    NULL,
+                                    NULL,
                                     NULL);
     gtk_signal_connect( GTK_OBJECT(about_dialog), "destroy",
 		        GTK_SIGNAL_FUNC(gtk_widget_destroyed), &about_dialog );
     gtk_widget_show(about_dialog);
     
-    
     return;
 }
