@@ -1,7 +1,7 @@
 #ifndef __WEATHER_H_
 #define __WEATHER_H_
 
-/* $Id: weather.h,v 1.7 2001/06/20 09:10:59 kmaraas Exp $ */
+/* $Id: weather.h,v 1.9 2002/01/23 16:07:49 kevinv Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -19,6 +19,8 @@
  * Location
  */
 
+G_BEGIN_DECLS
+
 #define WEATHER_LOCATION_NAME_MAX_LEN 100
 #define WEATHER_LOCATION_CODE_LEN     4
 #define WEATHER_LOCATION_ZONE_LEN     7
@@ -180,6 +182,10 @@ struct _WeatherInfo {
     WeatherPressure pressure;
     WeatherVisibility visibility;
     gchar *forecast;
+    gchar *metar_buffer;
+    gchar *iwin_buffer;
+    gchar *met_buffer;
+    gchar *radar_buffer;
     GdkPixmap *radar;
 };
 
@@ -221,6 +227,7 @@ extern void _weather_info_get_pixmap (WeatherInfo *info, gboolean mini, GdkPixma
 #define weather_info_get_pixmap_mini(info,pixmap,mask) _weather_info_get_pixmap((info), TRUE, (pixmap), (mask))
 #define weather_info_get_pixmap(info,pixmap,mask) _weather_info_get_pixmap((info), FALSE, (pixmap), (mask))
 
+G_END_DECLS
 
 #endif /* __WEATHER_H_ */
 
