@@ -1,4 +1,4 @@
-/*  panel-menu-directory.c
+/* panel-menu-directory.c
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Library General Public License as published by
@@ -19,6 +19,9 @@
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
 #include <panel-applet.h>
+#include <panel-applet-gconf.h>
+#include <gconf/gconf.h>
+#include <gconf/gconf-client.h>
 
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-init.h>
@@ -29,41 +32,50 @@
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-result.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs-monitor.h>
 
 #include "panel-menu.h"
 #include "panel-menu-common.h"
+#include "panel-menu-config.h"
+#include "panel-menu-pixbuf.h"
 #include "panel-menu-directory.h"
 
 static const gchar *directory_menu_xml =
 	"    <placeholder name=\"ChildItem\">\n"
 	"        <menuitem name=\"Action\" verb=\"Action\" label=\"%s Properties...\"\n"
-	"                  pixtype=\"stock\" pixname=\"gtk-close\"/>\n"
+	"                  pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
+	"%s"
 	"        <menuitem name=\"Remove\" verb=\"Remove\" label=\"Remove %s\"\n"
 	"                  pixtype=\"stock\" pixname=\"gtk-close\"/>\n"
 	"        <separator/>"
 	"    </placeholder>";
 
+static const gchar *additional_menu_xml =
+	"        <menuitem name=\"Regenerate\" verb=\"Regenerate\" label=\"Regenerate Menus\"\n"
+	"                  pixtype=\"stock\" pixname=\"gtk-refresh\"/>\n";
+
 typedef struct _PanelMenuDocuments {
-	GtkWidget *checkitem;
+	gint id;
 	GtkWidget *directory;
-	GtkWidget *regenitem;
 	GtkWidget *menu;
 	gchar *name;
 	gchar *path;
 	gint level;
-	time_t mtime;
-	gint timeout_id;
+	GnomeVFSMonitorHandle *monitor;
 } PanelMenuDirectory;
 
-static gint check_update_directory (PanelMenuEntry *entry);
-static time_t get_directory_mtime (gchar *uri);
-static void regenerate_menus_cb (GtkWidget *menuitem, PanelMenuEntry *entry);
+static void directory_changed_cb(GnomeVFSMonitorHandle *handle,
+				 const gchar *monitor_uri,
+				 const gchar *info_uri,
+				 GnomeVFSMonitorEventType event_type,
+				 gpointer user_data);
+static void regenerate_menus_cb (GtkWidget *menuitem, PanelMenuEntry *entry,
+				 const gchar *verb);
 static void panel_menu_directory_load (const gchar *uri, GtkMenuShell *parent,
 				       gint level);
 static void directory_load_cb (GnomeVFSAsyncHandle *handle,
 			       GnomeVFSResult result, GList *list,
 			       guint entries_read, GtkMenuShell *parent);
-static void set_visibility (GtkCheckMenuItem *checkitem, GtkWidget *target);
 static void change_directory_cb (GtkWidget *widget, PanelMenuEntry *entry,
 				 const gchar *verb);
 static GtkWidget *panel_menu_directory_edit_dialog_new (gchar *title,
@@ -71,53 +83,93 @@ static GtkWidget *panel_menu_directory_edit_dialog_new (gchar *title,
 							GtkWidget **pentry,
 							GtkWidget **spin);
 
+static gint object_counter = 0;
+
 PanelMenuEntry *
 panel_menu_directory_new (PanelMenu *parent, gchar *name, gchar *path,
 			  gint level)
+{
+	PanelMenuEntry *entry;
+	gchar *item_key;
+
+	item_key = g_strdup_printf ("directory%d/name", object_counter);
+	panel_applet_gconf_set_string (parent->applet, item_key, name, NULL);
+	g_free (item_key);
+	item_key = g_strdup_printf ("directory%d/path", object_counter);
+	panel_applet_gconf_set_string (parent->applet, item_key, path, NULL);
+	g_free (item_key);
+	item_key = g_strdup_printf ("directory%d/level", object_counter);
+	panel_applet_gconf_set_int (parent->applet, item_key, level, NULL);
+	g_free (item_key);
+	entry = panel_menu_directory_new_with_id (parent, object_counter);
+	return entry;
+}
+
+PanelMenuEntry *
+panel_menu_directory_new_with_id (PanelMenu *parent, gint id)
 {
 	PanelMenuEntry *entry;
 	PanelMenuDirectory *directory;
 	GtkWidget *tearoff;
 	GtkWidget *image;
-	GtkWidget *sep;
+	GConfClient *client;
+	gchar *base_key;
+	gchar *dir_key;
+	gchar *name;
+	gchar *path;
+	gint level;
+
+	if (id > object_counter)
+		object_counter = id;
 
 	entry = g_new0 (PanelMenuEntry, 1);
 	entry->type = PANEL_MENU_TYPE_DIRECTORY;
 	entry->parent = parent;
 	directory = g_new0 (PanelMenuDirectory, 1);
 	entry->data = (gpointer) directory;
-	directory->level = level;
+	directory->id = id;
 	directory->directory = gtk_menu_item_new_with_label ("");
 	panel_menu_common_widget_dnd_init (entry);
 	gtk_widget_show (directory->directory);
-	directory->checkitem = gtk_check_menu_item_new_with_label ("");
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM
-					(directory->checkitem), TRUE);
-	g_signal_connect (G_OBJECT (directory->checkitem), "toggled",
-			  G_CALLBACK (panel_menu_common_set_visibility),
-			  directory->directory);
 	directory->menu = gtk_menu_new ();
+
 	tearoff = gtk_tearoff_menu_item_new ();
 	gtk_menu_shell_append (GTK_MENU_SHELL (directory->menu), tearoff);
-	gtk_widget_show (tearoff);
-	directory->regenitem =
-		gtk_image_menu_item_new_with_label (_("Regenerate Menus"));
-	image = gtk_image_new_from_stock ("gtk-refresh", GTK_ICON_SIZE_MENU);
-	gtk_widget_show (image);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM
-				       (directory->regenitem), image);
-	gtk_menu_shell_append (GTK_MENU_SHELL (directory->menu),
-			       directory->regenitem);
-	g_signal_connect (G_OBJECT (directory->regenitem), "activate",
-			  G_CALLBACK (regenerate_menus_cb), entry);
-	gtk_widget_show (directory->regenitem);
-	sep = gtk_menu_item_new ();
-	gtk_menu_shell_append (GTK_MENU_SHELL (directory->menu), sep);
-	gtk_widget_show (sep);
+	if (parent->menu_tearoffs) {
+		gtk_widget_show (tearoff);
+	}
+
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (directory->directory),
 				   directory->menu);
+
+	client = gconf_client_get_default ();
+	base_key = g_strdup_printf ("directory%d", id);
+	dir_key = panel_applet_gconf_get_full_key (parent->applet, base_key);
+	if (gconf_client_dir_exists (client, dir_key, NULL)) {
+		gchar *key;
+		key = g_strdup_printf ("%s/name", base_key);
+		name = panel_applet_gconf_get_string (parent->applet, key, NULL);
+		g_free (key);
+		key = g_strdup_printf ("%s/path", base_key);
+		path = panel_applet_gconf_get_string (parent->applet, key, NULL);
+		g_free (key);
+		key = g_strdup_printf ("%s/level", base_key);
+		level = panel_applet_gconf_get_int (parent->applet, key, NULL);
+		g_free (key);
+	} else {
+		name = g_strdup ("My Documents");
+		path = g_strdup_printf ("%s/Documents", g_get_home_dir ());
+		level = 1;
+	}
+	g_object_unref (G_OBJECT (client));
+	g_free (base_key);
+	g_free (dir_key);
+	directory->level = level;
 	panel_menu_directory_set_name (entry, name);
 	panel_menu_directory_set_path (entry, path);
+	g_free (name);
+	g_free (path);
+	object_counter++;
 	return entry;
 }
 
@@ -125,7 +177,6 @@ void
 panel_menu_directory_set_name (PanelMenuEntry *entry, gchar *name)
 {
 	PanelMenuDirectory *directory;
-	gchar *show_label;
 
 	g_return_if_fail (entry != NULL);
 	g_return_if_fail (name != NULL);
@@ -137,10 +188,6 @@ panel_menu_directory_set_name (PanelMenuEntry *entry, gchar *name)
 	directory->name = name ? g_strdup (name) : g_strdup ("");
 	gtk_label_set_text (GTK_LABEL (GTK_BIN (directory->directory)->child),
 			    name);
-	show_label = g_strconcat ("Show ", directory->name, NULL);
-	gtk_label_set_text (GTK_LABEL (GTK_BIN (directory->checkitem)->child),
-			    show_label);
-	g_free (show_label);
 }
 
 void
@@ -157,86 +204,63 @@ panel_menu_directory_set_path (PanelMenuEntry *entry, gchar *path)
 	directory = (PanelMenuDirectory *) entry->data;
 	if (directory->path)
 		g_free (directory->path);
-	directory->path = g_strdup (path);
-	regenerate_menus_cb (NULL, entry);
-	directory->mtime = get_directory_mtime (directory->path);
-	panel_menu_directory_start_timeout (entry);
-}
-
-void
-panel_menu_directory_start_timeout (PanelMenuEntry *entry)
-{
-	PanelMenuDirectory *directory;
-
-	g_return_if_fail (entry != NULL);
-	g_return_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY);
-
-	directory = (PanelMenuDirectory *) entry->data;
-	panel_menu_directory_stop_timeout (entry);
-	if (entry->parent->auto_directory_update
-	    && entry->parent->auto_directory_update_timeout) {
-		directory->timeout_id =
-			gtk_timeout_add (entry->parent->
-					 auto_directory_update_timeout *1000,
-					 (GtkFunction) check_update_directory,
-					 entry);
-	}
-}
-
-void
-panel_menu_directory_stop_timeout (PanelMenuEntry *entry)
-{
-	PanelMenuDirectory *directory;
-
-	g_return_if_fail (entry != NULL);
-	g_return_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY);
-
-	directory = (PanelMenuDirectory *) entry->data;
-	if (directory->timeout_id)
-		gtk_timeout_remove (directory->timeout_id);
-	directory->timeout_id = 0;
-}
-
-static gint
-check_update_directory (PanelMenuEntry *entry)
-{
-	PanelMenuDirectory *directory;
-	time_t time;
-	gboolean retval;
-
-	g_return_if_fail (entry != NULL);
-	g_return_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY);
-
-	directory = (PanelMenuDirectory *) entry->data;
-	if (entry->parent->auto_directory_update) {
-		time = get_directory_mtime (directory->path);
-		if (time > directory->mtime) {
-			g_print ("directory modified, updating menu contents.\n");
-			regenerate_menus_cb (NULL, entry);
-			directory->mtime = time;
-		}
-		retval = TRUE;
-	} else {
-		panel_menu_directory_stop_timeout (entry);
-		retval = FALSE;
-	}
-	return retval;
-}
-
-static time_t
-get_directory_mtime (gchar *uri)
-{
-	struct stat s;
-	time_t mtime = 0;
-
-	if ((stat (uri, &s) == 0) && S_ISDIR (s.st_mode)) {
-		mtime = s.st_mtime;
-	}
-	return mtime;
+	if (directory->monitor)
+		gnome_vfs_monitor_cancel (directory->monitor);
+	directory->path = panel_menu_common_build_full_path (path, "");;
+	regenerate_menus_cb (NULL, entry, NULL);
+	gnome_vfs_monitor_add (&directory->monitor,
+			       directory->path,
+			       GNOME_VFS_MONITOR_DIRECTORY,
+			       directory_changed_cb,
+			       entry);
+	if (directory->monitor)
+		g_print ("monitor successfully installed for %s\n",
+			  directory->path);
+	else
+		g_print ("monitor installation failed for %s\n",
+			  directory->path);
 }
 
 static void
-regenerate_menus_cb (GtkWidget *menuitem, PanelMenuEntry *entry)
+directory_changed_cb(GnomeVFSMonitorHandle *handle,
+		     const gchar *monitor_uri,
+		     const gchar *info_uri,
+		     GnomeVFSMonitorEventType event_type,
+		     gpointer user_data)
+{
+	PanelMenuEntry *entry;
+	entry = (PanelMenuEntry *) user_data;
+
+	switch (event_type) {
+		case GNOME_VFS_MONITOR_EVENT_CHANGED:
+			g_print ("GNOME_VFS_MONITOR_EVENT_CHANGED");
+			break;
+		case GNOME_VFS_MONITOR_EVENT_DELETED:
+			g_print ("GNOME_VFS_MONITOR_EVENT_DELETED");
+			break;
+		case GNOME_VFS_MONITOR_EVENT_STARTEXECUTING:
+			g_print ("GNOME_VFS_MONITOR_EVENT_STARTEXECUTING");
+			break;
+		case GNOME_VFS_MONITOR_EVENT_STOPEXECUTING:
+			g_print ("GNOME_VFS_MONITOR_EVENT_STOPEXECUTING");
+			break;
+		case GNOME_VFS_MONITOR_EVENT_CREATED:
+			g_print ("GNOME_VFS_MONITOR_EVENT_CREATED");
+			break;
+		case GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED:
+			g_print ("GNOME_VFS_MONITOR_EVENT_METADATA_CHANGED");
+			break;
+		default:
+			break;
+	}
+	g_print (" (%s)", info_uri);
+	g_print ("\n");
+	regenerate_menus_cb (NULL, entry, NULL);
+}
+
+static void
+regenerate_menus_cb (GtkWidget *menuitem, PanelMenuEntry *entry,
+		     const gchar *verb)
 {
 	PanelMenuDirectory *directory;
 	GList *list;
@@ -252,11 +276,8 @@ regenerate_menus_cb (GtkWidget *menuitem, PanelMenuEntry *entry)
 			gtk_widget_destroy (GTK_WIDGET (cur->data));
 	}
 	g_list_free (list);
-	panel_menu_directory_stop_timeout (entry);
-	panel_menu_directory_load (directory->path,
-				   GTK_MENU_SHELL (directory->menu),
+	panel_menu_directory_load (directory->path, GTK_MENU_SHELL(directory->menu),
 				   directory->level);
-	panel_menu_directory_start_timeout (entry);
 }
 
 void
@@ -274,9 +295,15 @@ panel_menu_directory_merge_ui (PanelMenuEntry *entry)
 
 	bonobo_ui_component_add_verb (component, "Action",
 				     (BonoboUIVerbFn)change_directory_cb, entry);
+	bonobo_ui_component_add_verb (component, "Regenerate",
+				     (BonoboUIVerbFn)regenerate_menus_cb, entry);
 	bonobo_ui_component_add_verb (component, "Remove",
 				     (BonoboUIVerbFn)panel_menu_common_remove_entry, entry);
-	xml = g_strdup_printf (directory_menu_xml, directory->name, directory->name);
+	xml = g_strdup_printf (directory_menu_xml,
+			       directory->name,
+			       directory->monitor ?
+			       "" : additional_menu_xml,
+			       directory->name);
 	bonobo_ui_component_set (component, "/popups/button3/ChildMerge/",
 				 xml, NULL);
 	g_free (xml);
@@ -295,7 +322,8 @@ panel_menu_directory_destroy (PanelMenuEntry *entry)
 		g_free (directory->name);
 	if (directory->path)
 		g_free (directory->path);
-	gtk_widget_destroy (directory->checkitem);
+	if (directory->monitor)
+		gnome_vfs_monitor_cancel (directory->monitor);
 	gtk_widget_destroy (directory->directory);
 	g_free (directory);
 }
@@ -312,53 +340,6 @@ panel_menu_directory_get_widget (PanelMenuEntry *entry)
 	return directory->directory;
 }
 
-GtkWidget *
-panel_menu_directory_get_checkitem (PanelMenuEntry *entry)
-{
-	PanelMenuDirectory *directory;
-
-	g_return_val_if_fail (entry != NULL, NULL);
-	g_return_val_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY, NULL);
-
-	directory = (PanelMenuDirectory *) entry->data;
-	return directory->checkitem;
-}
-
-gchar *
-panel_menu_directory_dump_xml (PanelMenuEntry *entry)
-{
-	PanelMenuDirectory *directory;
-	GString *string;
-	gchar *str;
-	gboolean visible;
-	gchar *level;
-
-	g_return_val_if_fail (entry != NULL, NULL);
-	g_return_val_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY, NULL);
-
-	directory = (PanelMenuDirectory *) entry->data;
-	visible = GTK_CHECK_MENU_ITEM (directory->checkitem)->active;
-	level = g_strdup_printf ("%d", directory->level);
-
-	string = g_string_new ("    <directory-item>\n" "        <name>");
-	g_string_append (string, directory->name);
-	g_string_append (string, "</name>\n");
-	g_string_append (string, "        <level>");
-	g_string_append (string, level);
-	g_string_append (string, "</level>\n");
-	g_string_append (string, "        <path>");
-	g_string_append (string, directory->path);
-	g_string_append (string, "</path>\n");
-	g_string_append (string, "        <visible>");
-	g_string_append (string, visible ? "true" : "false");
-	g_string_append (string, "</visible>\n");
-	g_string_append (string, "    </directory-item>\n");
-	g_free (level);
-	str = string->str;
-	g_string_free (string, FALSE);
-	return str;
-}
-
 gboolean
 panel_menu_directory_accept_drop (PanelMenuEntry *entry, GnomeVFSURI *uri)
 {
@@ -434,9 +415,8 @@ directory_load_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 						  G_CALLBACK
 						  (panel_menu_common_destroy_apps_menuitem),
 						  NULL);
-				panel_menu_common_set_icon_scaled_from_file
-					(GTK_MENU_ITEM (menuitem),
-					 "/usr/share/pixmaps/gnome-folder.png");
+				panel_menu_pixbuf_set_icon (GTK_MENU_ITEM (menuitem),
+							   "directory");
 				gtk_menu_shell_append (parent, menuitem);
 				gtk_widget_show (menuitem);
 				submenu = gtk_menu_new ();
@@ -459,8 +439,7 @@ directory_load_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 								    name),
 								   "icon-filename");
 			if (icon) {
-				panel_menu_common_set_icon_scaled_from_file
-					(GTK_MENU_ITEM (menuitem), icon);
+				panel_menu_pixbuf_set_icon (GTK_MENU_ITEM (menuitem), icon);
 			} else {
 				image = gtk_image_new_from_stock (GTK_STOCK_NEW,
 								  GTK_ICON_SIZE_MENU);
@@ -529,13 +508,10 @@ panel_menu_directory_new_with_dialog (PanelMenu *panel_menu)
 						  level);
 		panel_menu->entries =
 			g_list_append (panel_menu->entries, (gpointer) entry);
-		panel_menu_options_append_option (panel_menu_common_find_options
-						  (panel_menu),
-						  panel_menu_directory_get_checkitem
-						  (entry));
 		gtk_menu_shell_append (GTK_MENU_SHELL (panel_menu->menubar),
 				       panel_menu_directory_get_widget (entry));
-		panel_menu_common_set_changed (panel_menu);
+		panel_menu_config_save_layout (panel_menu);
+		panel_menu_directory_save_config (entry);
 	}
 	gtk_widget_destroy (dialog);
 }
@@ -593,7 +569,7 @@ change_directory_cb (GtkWidget *widget, PanelMenuEntry *entry, const gchar *verb
 			changed = TRUE;
 		}
 		if (changed)
-			panel_menu_common_set_changed (entry->parent);
+			panel_menu_directory_save_config (entry);
 	}
 	gtk_widget_destroy (dialog);
 }
@@ -650,3 +626,59 @@ panel_menu_directory_edit_dialog_new (gchar *title, GtkWidget **nentry,
 	gtk_widget_show (*spin);
 	return dialog;
 }
+
+gchar *
+panel_menu_directory_save_config (PanelMenuEntry *entry)
+{
+	PanelMenuDirectory *directory;
+	PanelMenu *panel_menu;
+	PanelApplet *applet;
+	gchar *id;
+	gchar *key;
+
+	g_return_if_fail (entry != NULL);
+	g_return_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY);
+
+	panel_menu = entry->parent;
+	applet = panel_menu->applet;
+	directory = (PanelMenuDirectory *) entry->data;
+
+	id = g_strdup_printf ("directory%d", directory->id);
+	key = g_strdup_printf ("%s/name", id);
+	panel_applet_gconf_set_string (applet, key, directory->name, NULL);
+	g_free (key);
+	key = g_strdup_printf ("%s/path", id);
+	panel_applet_gconf_set_string (applet, key, directory->path, NULL);
+	g_free (key);
+	key = g_strdup_printf ("%s/level", id);
+	panel_applet_gconf_set_int (applet, key, directory->level, NULL);
+	g_free (key);
+	return id;
+}
+
+void
+panel_menu_directory_remove_config (PanelMenuEntry *entry)
+{
+	PanelMenuDirectory *directory;
+	PanelMenu *panel_menu;
+	PanelApplet *applet;
+	GConfClient *client;
+	gchar *base_key;
+	gchar *key;
+
+	g_return_if_fail (entry != NULL);
+	g_return_if_fail (entry->type == PANEL_MENU_TYPE_DIRECTORY);
+
+	panel_menu = entry->parent;
+	applet = panel_menu->applet;
+	directory = (PanelMenuDirectory *) entry->data;
+
+	client = gconf_client_get_default ();
+	g_return_if_fail (client != NULL);
+	base_key = panel_applet_get_preferences_key (applet);
+	key = g_strdup_printf ("%s/directory%d", base_key, directory->id);
+	_gconf_client_clean_dir (client, key);
+	g_free (base_key);
+	g_free (key);
+	g_object_unref (G_OBJECT (client));
+}
